/**
 *******************************************************************************
 * @file        a31t21x_debug.c
 * @brief       Application common
 * @version
 * @date
 * @author
 * @note
 *
 * Copyright(C) 2017, ABOV Semiconductor
 * All rights reserved.
 *
 ******************************************************************************/

#include <stdint.h>
#include <stddef.h>

#include "a31t21x_system.h"
#include "a31t21x_gpio.h"
#include "a31t21x_uart.h"
#include "a31t21x_usart1n.h"

#include "debug_serial.h"
#include "debug_log.h"
#include "main.h"

#define USE_UART_DEBUG		// Select 'USE_UART_DEBUG' or 'USE_USART_DEBUG'

#ifdef	USE_UART_DEBUG
#define DEBUG_UART_ID   UART_ID_0
#else
#define DEBUG_USART_ID	USART_ID_10
#endif

#ifdef USE_UART_DEBUG
static void dbg_uart_irq_handler(uint32_t event, void *context)
{
    if (event & UART_EVENT_RECEIVE_DONE)
    {
        serial_continue_rx();
    }

    if (event & UART_EVENT_TRANSMIT_DONE)
    {
        serial_continue_tx();
    }
}
#else
static void app_usart_irq_handler(uint32_t event, void *context)
{
    if (event & USART_EVENT_RECEIVE_DONE)
    {
        serial_continue_rx();
    }

    if (event & USART_EVENT_TRANSMIT_DONE)
    {
        serial_continue_tx();
    }
}
#endif

static void dbg_serial_init(void)
{
#ifdef USE_UART_DEBUG
    struct uart_cfg cfg =
    {
        .baudrate = APP_UART_BAUD,
        .data_bit = UART_DATA_BIT_8,
        .parity_bit = UART_PARITY_BIT_NONE,
        .stop_bit = UART_STOP_BIT_1,
        .irq_prio = DRIVER_PRIORITY_LOWEST,
    };

    HAL_UART_Init(DEBUG_UART_ID, &cfg, dbg_uart_irq_handler, NULL);
#else
    struct usart_cfg cfg =
    {
				.mode = USART_MODE_ASYNC,
        .baudrate = APP_UART_BAUD,
        .data_bit = USART_DATA_BIT_8,
        .parity_bit = USART_PARITY_BIT_NONE,
        .stop_bit = USART_STOP_BIT_1,
        .irq_prio = DRIVER_PRIORITY_LOWEST,
    };

    HAL_USART_Init(USART_ID_10, &cfg, app_usart_irq_handler, NULL);
#endif
}

static void dbg_serial_abort(void)
{
#ifdef USE_UART_DEBUG
    HAL_UART_Abort(DEBUG_UART_ID);
#else
#endif
}

static void dbg_serial_rx(uint8_t *ch)
{
#ifdef USE_UART_DEBUG
    HAL_UART_Receive(DEBUG_UART_ID, ch, 1, UART_IO_TYPE_NON_BLOCK);
#else
		HAL_USART_Receive(DEBUG_USART_ID, ch, 1, USART_IO_TYPE_NON_BLOCK);
#endif
}

static void dbg_serial_rx_poll(uint8_t *ch)
{
#ifdef USE_UART_DEBUG
    HAL_UART_Receive(DEBUG_UART_ID, ch, 1, UART_IO_TYPE_BLOCK);
#else
		HAL_USART_Receive(DEBUG_USART_ID, ch, 1, USART_IO_TYPE_BLOCK);
#endif
}

static void dbg_serial_tx(uint8_t *ch)
{
#ifdef USE_UART_DEBUG
    HAL_UART_Transmit(DEBUG_UART_ID, ch, 1, UART_IO_TYPE_NON_BLOCK);
#else
    HAL_USART_Transmit(DEBUG_USART_ID, ch, 1, USART_IO_TYPE_NON_BLOCK);
#endif
}

static void dbg_serial_tx_poll(uint8_t *ch)
{
#ifdef USE_UART_DEBUG
    HAL_UART_Transmit(DEBUG_UART_ID, ch, 1, UART_IO_TYPE_BLOCK);
#else
    HAL_USART_Transmit(DEBUG_USART_ID, ch, 1, USART_IO_TYPE_BLOCK);
#endif
}

struct serial_op s_op =
{
    .init = dbg_serial_init,
    .abort = dbg_serial_abort,
    .rx = dbg_serial_rx,
    .rx_poll = dbg_serial_rx_poll,
    .tx = dbg_serial_tx,
    .tx_poll = dbg_serial_tx_poll,
};

void Debug_Init(void)
{
#ifdef USE_UART_DEBUG
    /* Configure UART TXD0 pin to be used */
    HAL_GPIO_SetFunction(PORTB, PIN4, Pn_MUX_AF1);
    HAL_GPIO_SetMode(PORTB, PIN4, GPIO_MODE_FUNC);
    HAL_GPIO_SetPull(PORTB, PIN4, GPIO_PULL_UP);
    /* Configure UART RXD0 pin to be used */
    HAL_GPIO_SetFunction(PORTB, PIN5, Pn_MUX_AF1);
    HAL_GPIO_SetMode(PORTB, PIN5, GPIO_MODE_FUNC);
    HAL_GPIO_SetPull(PORTB, PIN5, GPIO_PULL_UP);
#else
    /* Configure UART TXD10 pin to be used */
    HAL_GPIO_SetFunction(PORTB, PIN0, Pn_MUX_AF1);
    HAL_GPIO_SetMode(PORTB, PIN0, GPIO_MODE_FUNC);
    HAL_GPIO_SetPull(PORTB, PIN0, GPIO_PULL_UP);
    /* Configure UART RXD10 pin to be used */
    HAL_GPIO_SetFunction(PORTB, PIN1, Pn_MUX_AF1);
    HAL_GPIO_SetMode(PORTB, PIN1, GPIO_MODE_FUNC);
    HAL_GPIO_SetPull(PORTB, PIN1, GPIO_PULL_UP);
#endif

    serial_init(&s_op);
}

void Assert(char *file, uint32_t line)
{
    LOG("ASSERT %s at %d\n", file, line);
    while (1)
    {
    }
}

